function [YPSNR, YSSIM] = STGSM(orginal_sequence, noisy_sequnce, denoised_sequence, result_file, width, height, sig, frmDepth, totalframes);

%Denoise image sequence using GSM model
%		fh = STGSM(orginal_sequence, noisy_sequnce, denoised_sequence, result_file, width, height, sig, frmDepth, totalframes);
%       orginal_sequence :Path to the original YUV 4:2:0 sequence.
%       noisy_sequence   :Path to the noisy YUV sequence.
%       denoised_sequence:Path to the output denoised sequence.
%       result_file      :Path to results file.
%       width            :Width of a frame.
%       height           :Heightof a frame.
%       sig              :Noise sigma.
%       frmdepth         :Number of frames in one direction to be used for denoising.
%       totalframes      :Total number of frames in the sequence to be denoised.
%
%       YPSNR            :PSNR results of all frames
%       YSSIM            :SSIM results of all frames


    forg = fopen(orginal_sequence,'r');
    fid = fopen(noisy_sequnce,'r');
    fout = fopen(denoised_sequence,'w');
    fresult = fopen(result_file,'w');
    
    
    
    
    %YUV format (4:2:0 in this case)
    y_elements= width*height;
    uv_elements = width*height/4;

    Ny = height;
    Nx = width;

    PS = ones(Ny,Nx);	% power spectral density (in this case, flat, i.e., white noise)

    % Pyramidal representation parameters
    Nsc = ceil(log2(min(Ny,Nx)) - 4);  % Number of scales (adapted to the image size)

    Nor = 8;                           % 8 orientations
    repres1 = 'fs';                    % Full Steerable Pyramid, 5 scales for 512x512
    repres2 = '';                      % Dummy parameter when using repres1 = 'fs'   
    parent = 0;      

    % Model parameters (optimized: do not change them unless you are an advanced user with a deep understanding of the theory)
    blSize = [3 3];	    % n x n coefficient neighborhood of spatial neighbors within the same subband
                        % (n must be odd): 
                        % including or not (1/0) in the neighborhood a coefficient from the same spatial location
                        % and orientation as the central coefficient of the n x n neighborhood, but
                        % next coarser scale. Many times helps, but not always.
    boundary = 1;		% Boundary mirror extension, to avoid boundary artifacts 
    covariance = 1;     % Full covariance matrix (1) or only diagonal elements (0).
    optim = 1;          % Bayes Least Squares solution (1), or MAP-Wiener solution in two steps (0)


    if ~exist('blSize'),
        blSzX = 3;	% Block size
        blSzY = 3;
    else
        blSzY = blSize(1);
        blSzX = blSize(2);
    end

    if (blSzY/2==floor(blSzY/2))|(blSzX/2==floor(blSzX/2)),
       error('Spatial dimensions of neighborhood must be odd!');
    end   

    no_white = 0;   % Power spectral density of noise. Default is white noise

    if ~exist('parent'),
        parent = 1;
    end

    if ~exist('boundary'),
        boundary = 1;
    end

    if ~exist('Nsc'),
        Nsc = 4;
    end

    if ~exist('Nor'),
        Nor = 8;
    end

    if ~exist('covariance'),
            covariance = 1;
    end

    if ~exist('optim'),
            optim = 1;
    end

    if ~exist('repres1'),
            repres1 = 'fs';
    end

    if ~exist('repres2'),
            repres2 = '';
    end

    if (((repres1=='w') | (repres1=='uw')) & (Nor~=3)),
        warning('For X-Y separable representations Nor must be 3. Nor = 3 assumed.');
        Nor = 3;
    end    


    [ydelta,yPS] = gen_Delta(Ny, Nx, blSzY, blSzX, Nsc, sig, no_white, boundary, repres1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    numFrames=0;

    ypyr =[];
    ypind =[];

    im_y = zeros(height, width, 2*frmDepth+1);

    for j=frmDepth+2:2*frmDepth+1
        [ycur,count] = fread(fid,y_elements,'uchar');
        ycur=double(reshape(ycur,[width,height])');
        [ucur,count] = fread(fid,uv_elements,'uchar');
        [vcur,count] = fread(fid,uv_elements,'uchar');

        im_y(:,:,j) = ycur;
        im_u(:,:,j) = ucur;
        im_v(:,:,j) = vcur;
     end

    numFrames = frmDepth;
    weight = ones(1,2*frmDepth+1);
    repFrames = 0;
    
    for i=1:totalframes

        fprintf(1,'Frame:%d\n',i);
        
        
        %reading next frame
        if( frmDepth+i <= totalframes)
            [ycur,count] = fread(fid,y_elements,'uchar');
            ycur=double(reshape(ycur,[width,height])');
            [ucur,count] = fread(fid,uv_elements,'uchar');
            [vcur,count] = fread(fid,uv_elements,'uchar');
        else
            repFrames = repFrames + 1 ;
        end
        
        
        %reading original frame
        [yorg,count] = fread(forg,y_elements,'uchar');
        yorg = double(reshape(yorg,[width,height])');
        [uorg,count] = fread(forg,uv_elements,'uchar');
        [vorg,count] = fread(forg,uv_elements,'uchar');
       
        for j=1:2*frmDepth
            im_y(:,:,j) = im_y(:,:,j+1);
            im_u(:,:,j) = im_u(:,:,j+1);
            im_v(:,:,j) = im_v(:,:,j+1);
        end

        im_y(:,:,2*frmDepth+1) = ycur;
        im_u(:,:,2*frmDepth+1) = ucur;
        im_v(:,:,2*frmDepth+1) = vcur;

        [im, Npy, Npx, By, Bx, Bpy, Bpx] = extend_Image(im_y(:,:,frmDepth+1), blSzY, blSzX, Nsc, boundary, repres1, repres2);
        [ypyr(:,1), ypind(:,:,1)] = buildFullSFpyr2(im, Nsc, Nor-1);  
        ysiz = [Ny Nx Npy Npx By Bx Bpy Bpx ];


        iCount = 1;
        
        if numFrames < 2*frmDepth+1
            numFrames = numFrames+1;
        end
            
        for j=2*frmDepth+2-numFrames:2*frmDepth+1 - repFrames
            if(j ~= frmDepth+1)
                iCount = iCount+1;
                %global motion compensation
                [ushift, vshift] = globalMV_NRCC(im_y(:,:,j), im_y(:,:,frmDepth+1), sig);
                im = warpImage(im_y(:,:,j), ushift, vshift);
                [im, Npy, Npx, By, Bx, Bpy, Bpx] = extend_Image(im, blSzY, blSzX, Nsc, boundary, repres1, repres2);
                [ypyr(:,iCount), ypind(:,:,iCount)] = buildFullSFpyr2(im.*weight(j), Nsc, Nor-1);
            end    
        end
        
        clear im iCount;
        tic; [im_d_y, ypyr, ypind] = denoi_BLS_GSM_video(sig, yPS, blSize, parent,  Nsc, Nor, covariance, optim, repres1, repres2, numFrames-repFrames, frmDepth, ydelta, ysiz, ypyr, ypind); toc


        %Rounding and bounds checking
        im_d_y =  double(uint8(im_d_y + 0.5));
        im_d_y = max(0,im_d_y);im_d_y = min(255,im_d_y);

        YPSNR(i) = 10*log10(255^2/mean2((yorg-double(uint8(im_d_y))).^2));
        YSSIM(i) = ssim_index(yorg,im_d_y);
        fprintf(fresult, 'YPSNR:%f YSSIM:%f\n', YPSNR(i), YSSIM(i));   
        fprintf(1, 'YPSNR:%f YSSIM:%f\n', YPSNR(i), YSSIM(i));   


        im_d_y = reshape(im_d_y',1,width*height);
        fwrite(fout,uint8(im_d_y),'uchar');
        fwrite(fout,im_u(:,:,frmDepth+1),'uchar');
        fwrite(fout,im_v(:,:,frmDepth+1),'uchar');
        %fwrite(fout,uint8(ucur),'uchar');
        %fwrite(fout,uint8(vcur),'uchar');
        clear im_d_y;

    end
    fclose(fid);
    fclose(fout);
    fclose(forg);
    fclose(fresult);

    clear im_y im_u im_v;;